<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xs="http://www.w3.org/2001/XMLSchema"
	exclude-result-prefixes="#all"
	version="2.0">
	
	<xsl:output method="xml"/>
	<xsl:output indent="yes"/>
	<xsl:strip-space elements="*"/>
	
	<xsl:variable name="nodeunitcodevalue"><xsl:text>{nodes}</xsl:text></xsl:variable>
	
	<xsl:template match="/">
		
		<xsl:for-each select="annotations/annotation">
			<!--<xsl:variable name="recordcount" select="position()"/>-->	<!-- need a numeric value for UID creation, which researchid is not always -->
			<xsl:variable name="order" select="order"/>
			<xsl:variable name="filename">annotationxmlfiles/<xsl:value-of select="$order"/>.xml</xsl:variable>
			<xsl:choose>
				<!--<xsl:when test="not(doc-available($filename))">-->	<!-- does not work -->
				<xsl:when test="not(unparsed-text-available($filename))">
					<xsl:result-document method="xml" indent="yes" href="{$filename}">
						<!--<xsl:message>order <xsl:value-of select="$order"/></xsl:message>-->
						
						<DicomStructuredReport>
							<DicomStructuredReportHeader>
								
								<SOPClassUID element="0016" group="0008" vr="UI">
									<value number="1">1.2.840.10008.5.1.4.1.1.88.22</value>	<!-- Enhanced SR is sufficient; do not need Comprehensive -->
								</SOPClassUID>
								
								<StudyInstanceUID element="000d" group="0020" vr="UI">
									<value number="1"><xsl:value-of select="studyinstanceuid"/></value>
								</StudyInstanceUID>
								
								<SeriesInstanceUID element="000e" group="0020" vr="UI">
									<value number="1">1.3.6.1.4.1.5962.1.3.0.0.1535642467.22502.<xsl:value-of select="$order"/></value>	<!-- from dummy dcsmpte - REPLACE :( -->
								</SeriesInstanceUID>
								
								<SOPInstanceUID element="0018" group="0008" vr="UI">
									<value number="1">1.3.6.1.4.1.5962.1.1.0.0.0.1535642467.22502.<xsl:value-of select="$order"/></value>	<!-- from dummy dcsmpte - REPLACE :( -->
								</SOPInstanceUID>
								
								<PatientName element="0010" group="0010" vr="PN">
									<value number="1"><xsl:value-of select="patientid"/></value>
								</PatientName>
								<PatientID element="0020" group="0010" vr="LO">
									<value number="1"><xsl:value-of select="patientid"/></value>
								</PatientID>
								<PatientBirthDate element="0030" group="0010" vr="DA"/>
								<PatientSex element="0040" group="0010" vr="CS">
									<value number="1"></value>
								</PatientSex>
								
								<StudyDate element="0020" group="0008" vr="DA">
									<value number="1"><xsl:value-of select="studydate"/></value>
								</StudyDate>
								<StudyTime element="0030" group="0008" vr="TM">
									<value number="1"><xsl:value-of select="studytime"/></value>
								</StudyTime>
								<ReferringPhysicianName element="0090" group="0008" vr="PN" />
								<StudyID element="0010" group="0020" vr="SH"/>
								<AccessionNumber element="0050" group="0008" vr="SH" />
								<StudyDescription element="1030" group="0008" vr="LO">
									<value number="1"><xsl:value-of select="anatomy"/></value>
								</StudyDescription>
								
								<Modality element="0060" group="0008" vr="CS">
									<value number="1">SR</value>
								</Modality>
								<SeriesNumber element="0011" group="0020" vr="IS">
									<value number="1">4578</value>
								</SeriesNumber>
								<SeriesDate element="0021" group="0008" vr="DA"/>
								<SeriesDescription element="103E" group="0008" vr="LO">
									<value number="1">Crowds Cure Cancer Annotation as Measurement Report</value>
								</SeriesDescription>
								<ReferencedPerformedProcedureStepSequence element="1111" group="0008" vr="SQ" />
								
								<Manufacturer element="0070" group="0008" vr="LO">
									<value number="1">PixelMed</value>
								</Manufacturer>
								<InstitutionName element="0080" group="0008" vr="LO"/>
								<StationName element="1010" group="0008" vr="SH">
									<value number="1">NONE</value>
								</StationName>
								<ManufacturerModelName element="1090" group="0008" vr="LO">
									<value number="1">XSLT from annotations_expanded.csv</value>
								</ManufacturerModelName>
								<DeviceSerialNumber element="1000" group="0018" vr="LO">
									<value number="1">9723613413261</value>
								</DeviceSerialNumber>
								<SoftwareVersions element="1020" group="0018" vr="LO">
									<value number="1">0.1</value>
								</SoftwareVersions>
								
								<InstanceNumber element="0013" group="0020" vr="IS">
									<value number="1">1</value>
								</InstanceNumber>
								<CompletionFlag element="a491" group="0040" vr="CS">
									<value number="1">COMPLETE</value>
								</CompletionFlag>
								<VerificationFlag element="a493" group="0040" vr="CS">
									<value number="1">UNVERIFIED</value>
								</VerificationFlag>
								
								<!-- https://stackoverflow.com/questions/32975793/datetime-to-epoch-and-vice-versa-xslt -->
								<xsl:variable name="contentdatetime"><xsl:value-of select="xs:dateTime('1970-01-01T00:00:00') + xs:dayTimeDuration(concat('PT', dateunix, 'S'))"/></xsl:variable>
								<!-- e.g., "2017-11-26T17:15:16" -->
								<!--<xsl:message>Converted dateunix is dateunix <xsl:value-of select="$contentdatetime"/></xsl:message> -->
								<ContentDate group="0008" element="0023" vr="DA"><value number="1"><xsl:value-of select="replace(substring($contentdatetime,1,10),'-','')"/></value></ContentDate>
								<ContentTime group="0008" element="0033" vr="TM"><value number="1"><xsl:value-of select="replace(substring($contentdatetime,12,8),':','')"/></value></ContentTime>

								<AuthorObserverSequence group="0040" element="a078" vr="SQ">
									<Item>
										<ObserverType group="0040" element="a084" vr="CS"><value number="1">PSN</value></ObserverType>
										<PersonName group="0040" element="a123" vr="PN"><value number="1"><xsl:value-of select="annotator"/></value></PersonName>
										<PersonIdentificationCodeSequence group="0040" element="1101" vr="SQ"/> <!-- Type 2C required if PSN -->
										<InstitutionName group="0008" element="0080" vr="LO"/> <!-- Type 2 -->
										<InstitutionCodeSequence group="0008" element="0082" vr="SQ"/> <!-- Type 2 -->
									</Item>
								</AuthorObserverSequence>
								
								<CurrentRequestedProcedureEvidenceSequence group="0040" element="a375" vr="SQ">
									<Item>
										<ReferencedSeriesSequence group="0008" element="1115" vr="SQ">
											<Item>
												<ReferencedSOPSequence group="0008" element="1199" vr="SQ">
													<Item>
														<ReferencedSOPClassUID group="0008" element="1150" vr="UI"><value number="1"><xsl:value-of select="sopclassuid"/></value></ReferencedSOPClassUID>
														<ReferencedSOPInstanceUID group="0008" element="1155" vr="UI"><value number="1"><xsl:value-of select="instanceuid"/></value></ReferencedSOPInstanceUID>
													</Item>
												</ReferencedSOPSequence>
												<SeriesInstanceUID group="0020" element="000e" vr="UI"><value number="1"><xsl:value-of select="seriesuid"/></value></SeriesInstanceUID>
											</Item>
										</ReferencedSeriesSequence>
										<StudyInstanceUID group="0020" element="000d" vr="UI"><value number="1"><xsl:value-of select="studyinstanceuid"/></value></StudyInstanceUID>
									</Item>
								</CurrentRequestedProcedureEvidenceSequence>
								
								<PerformedProcedureCodeSequence element="a372" group="0040" vr="SQ" />
								
							</DicomStructuredReportHeader>
							<DicomStructuredReportContent>
								<container continuity="SEPARATE" sopclass="1.2.840.10008.5.1.4.1.1.88.22" template="1500" templatemappingresource="DCMR">
									
									<concept cm="Imaging Measurement Report" csd="DCM" cv="126000" />
									
									<code relationship="HAS CONCEPT MOD">
										<concept cm="Language of Content Item and Descendants" csd="DCM" cv="121049" />
										<value cm="English" csd="RFC5646" cv="eng" />
										<code relationship="HAS CONCEPT MOD">
											<concept cm="Country of Language" csd="DCM" cv="121046" />
											<value cm="United States" csd="ISO3166_1" cv="US" />
										</code>
									</code>
									
									<pname relationship="HAS OBS CONTEXT"><concept cm="Person Observer Name" csd="DCM" cv="121008"/><value><xsl:value-of select="annotator"/></value></pname>
									
									<code relationship="HAS CONCEPT MOD">
										<concept cm="Procedure reported" csd="DCM" cv="121058"/>
										<xsl:choose>
											<xsl:when test="anatomy = 'Liver'">
												<value cm="CT Abdomen" csd="LN" cv="41806-1"/>
											</xsl:when>
											<xsl:when test="anatomy = 'Lung'">
												<value cm="CT Chest" csd="LN" cv="24627-2"/>
											</xsl:when>
											<xsl:when test="anatomy = 'Renal'">
												<value cm="CT Abdomen" csd="LN" cv="41806-1"/>
											</xsl:when>
											<xsl:when test="anatomy = 'Ovarian'">
												<value cm="CT Pelvis" csd="LN" cv="24865-8"/>
											</xsl:when>
											<xsl:otherwise>
												<xsl:message>ERROR - unrecognized anatomy <xsl:value-of select="anatomy"/></xsl:message>
											</xsl:otherwise>
										</xsl:choose>
									</code>
									
									<container continuity="SEPARATE" relationship="CONTAINS">
										<concept cm="Image Library" csd="DCM" cv="111028" />
										<container continuity="SEPARATE" relationship="CONTAINS">
											<concept cm="Image Library Group" csd="DCM" cv="126200" />
											<image relationship="CONTAINS">
												<class><xsl:value-of select="sopclassuid"/></class>
												<instance><xsl:value-of select="instanceuid"/></instance>
												
												<!-- per PS3.21 Section A.6.1.2 Content Tree - TID 1602 Mapping of Image Library Entry Descriptors-->
												
												<code relationship="HAS ACQ CONTEXT">
													<concept cm="Modality" csd="DCM" cv="121139" />
													<value cm="Computed Tomography" csd="DCM" cv="CT" />
												</code>
												
												<date relationship="HAS ACQ CONTEXT"><concept cm="Study Date" csd="DCM" cv="111060"/><value><xsl:value-of select="studydate"/></value></date>
												<time relationship="HAS ACQ CONTEXT"><concept cm="Study Time" csd="DCM" cv="111061"/><value><xsl:value-of select="135823"/></value></time>
											</image>
										</container>
									</container>
									
									<container continuity="SEPARATE" relationship="CONTAINS">
										<concept cm="Imaging Measurements" csd="DCM" cv="126010" />
										<container continuity="SEPARATE" relationship="CONTAINS">
											<concept cm="Measurement Group" csd="DCM" cv="125007" />
											
											<text relationship="HAS OBS CONTEXT"><concept cm="Tracking Identifier" csd="DCM" cv="112039"/><value><xsl:value-of select="id"/></value></text>
											<uidref relationship="HAS OBS CONTEXT"><concept cm="Tracking Unique Identifier" csd="DCM" cv="112040"/><value>1.3.6.1.4.1.5962.1.1.0.0.0.1535644357.22655.<xsl:value-of select="$order"/></value></uidref>	<!-- from (different) dummy dcsmpte - REPLACE :( -->
											
											<code relationship="HAS CONCEPT MOD">
												<concept cm="Finding Site" csd="SRT" cv="G-C0E3"/>
												<xsl:choose>
													<xsl:when test="anatomy = 'Liver'">
														<value cm="Liver" csd="SRT" cv="T-62000"/>
													</xsl:when>
													<xsl:when test="anatomy = 'Lung'">
														<value cm="Lung" csd="SRT" cv="T-28000"/>
													</xsl:when>
													<xsl:when test="anatomy = 'Renal'">
														<value cm="Kidney" csd="SRT" cv="T-71000"/>
													</xsl:when>
													<xsl:when test="anatomy = 'Ovarian'">
														<value cm="Ovary" csd="SRT" cv="T-87000"/>
													</xsl:when>
													<xsl:otherwise>
														<xsl:message>ERROR - unrecognized anatomy <xsl:value-of select="anatomy"/></xsl:message>
													</xsl:otherwise>
												</xsl:choose>
											</code>
											
											<num relationship="CONTAINS">
												<concept cm="Length" csd="SRT" cv="G-D7FE"/>
												<value><xsl:value-of select="length"/></value>
												<units cm="mm" csd="UCUM" cv="mm" />
												
												<scoord relationship="INFERRED FROM">
													<polyline>
														<x><xsl:value-of select="startx"/></x>
														<y><xsl:value-of select="starty"/></y>
														<x><xsl:value-of select="endx"/></x>
														<y><xsl:value-of select="endy"/></y>
													</polyline>
													<image relationship="SELECTED FROM">
														<class><xsl:value-of select="sopclassuid"/></class>
														<instance><xsl:value-of select="instanceuid"/></instance>
													</image>
												</scoord>
												
											</num>

										</container>
									</container>



									
									
									
									
									
									
									
								</container>
							</DicomStructuredReportContent>
						</DicomStructuredReport>
						
						
					</xsl:result-document>
					<xsl:choose>
						<!--<xsl:when test="doc-available($filename)">--> 	<!-- does not work -->
						<xsl:when test="unparsed-text-available($filename)">
							<xsl:message>Successfully wrote <xsl:value-of select="$filename"/></xsl:message>
						</xsl:when>
						<xsl:otherwise>
							<xsl:message>WARNING - Unsuccessfully wrote <xsl:value-of select="$filename"/></xsl:message>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>
					<xsl:message>WARNING - Have already done <xsl:value-of select="$filename"/> - assume duplicate row and same values :(</xsl:message>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>
	
</xsl:stylesheet>

